"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _utilsLogging = require("../../../utils/logging");

var _tasks = require("./tasks");

var _plugins = require("../plugins");

var _plugins2 = _interopRequireDefault(_plugins);

/*
    Hookable Build Pipeline Events
    ------------------------------
    To hook these events, place plugins in the following directory names
    under the dir_custom_tasks/production directory. Main tasks should not be under a specific
    sub-directory.

    - on-start
    - on-complete

    example: dir_custom_tasks/production/on-start/*.js will be run "on start".
*/

var getStandardBuild = function getStandardBuild(buildName, fn, cbOnComplete) {
    return _asyncToGenerator(function* (siteConfig) {

        var tasks = yield fn(siteConfig, _plugins2["default"]);

        var startTime = Date.now();

        var logger = (0, _utilsLogging.getLogger)(siteConfig.quiet, buildName);

        var customTasks = yield (0, _tasks.getCustomTasks)(siteConfig, _plugins2["default"]);

        if (customTasks) yield (0, _tasks.runTasks)(customTasks["on-start"]);

        var onComplete = _asyncToGenerator(function* () {
            if (customTasks) yield (0, _tasks.runTasks)(customTasks["on-complete"]);

            if (cbOnComplete) {
                yield cbOnComplete(siteConfig);
            }

            var endTime = Date.now();
            logger("Build " + buildName + " took " + (endTime - startTime) / 1000 + " seconds.");
        });

        try {
            yield (0, _tasks.runTasks)(tasks, siteConfig.source, onComplete, siteConfig.watch);
        } catch (ex) {
            console.log(ex);
            console.log(ex.stack);
        }
    });
};

exports["default"] = getStandardBuild;
module.exports = exports["default"];