"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var _jsYaml = require("js-yaml");

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _utilsConfig = require("../../utils/config");

var _utilsConfig2 = _interopRequireDefault(_utilsConfig);

var _utilsFs = require("../../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _utilsLogging = require("../../utils/logging");

var _utilsFileReader = require("../../utils/file-reader");

var _utilsFileReader2 = _interopRequireDefault(_utilsFileReader);

var _utilsCli = require("../../utils/cli");

var _utilsCli2 = _interopRequireDefault(_utilsCli);

var _buildsProduction = require("./builds/production");

var _buildsProduction2 = _interopRequireDefault(_buildsProduction);

var _buildsDebug = require("./builds/debug");

var _buildsDebug2 = _interopRequireDefault(_buildsDebug);

var _buildsDev = require("./builds/dev");

var _buildsDev2 = _interopRequireDefault(_buildsDev);

var _buildsStatic = require("./builds/static");

var _buildsStatic2 = _interopRequireDefault(_buildsStatic);

var _buildsDatabase = require("./builds/database");

var _buildsDatabase2 = _interopRequireDefault(_buildsDatabase);

var _buildUtilsTasks = require("./build-utils/tasks");

var _plugins = require("./plugins");

var _plugins2 = _interopRequireDefault(_plugins);

//modes

var _configurationsDefault = require("../../configurations/default");

var _configurationsDefault2 = _interopRequireDefault(_configurationsDefault);

var configurations = {
    "default": _configurationsDefault2["default"]
};

var builds = {
    "production": _buildsProduction2["default"],
    "debug": _buildsDebug2["default"],
    "dev": _buildsDev2["default"],
    "static": _buildsStatic2["default"],
    "database": _buildsDatabase2["default"]
};

var argv = _optimist2["default"].argv;

var printSyntax = function printSyntax(msg) {
    if (msg) {
        (0, _utilsLogging.print)("Error: " + msg);
    }
    (0, _utilsLogging.print)("Usage: fora build [-t <type>] [-s <source>] [-d <destination>]");
    process.exit();
};

var getArgs = function getArgs() {
    var args = _utilsCli2["default"].getArgs();

    /* params */
    var buildType = args.length >= 3 ? args[3] : null;
    return { buildType: buildType };
};

/*
    We have these build modes

    1. Production
        build-type production

        We generate server files and client_js files, and skip dev_js files.
        Source-maps are off by default. By default, we compile client_js files
        with full ES5 compatibility.

    2. Debug Build
        build-type debug

        We generate server files and client_js files, and skip dev_js files.
        Source-maps are on by default. We also compile client_js files
        without regenerator transforms.

    3. Dev Build
        build-type dev

        We do not transpile server js files, since the app will entirely run
        on the client. Source maps are on, and regenerator transforms are off.

    4. Static build
        build-type static

        We transpile server js files. We build static html files for each route.
        Client_js files will be transpiled (wil regenerator transforms) and
        source-maps will be on by default.

    5.  Database build
        build-type database

        Loads the build from siteConfig.custom_build_dir


    5.  Custom named build
        build-type "buildname"

        Loads the build from siteConfig.custom_build_dir

*/

var build = _asyncToGenerator(function* () {
    var siteConfig = yield getSiteConfig(argv.s || argv.source || "./", argv.d || argv.destination || "_site");
    var logger = (0, _utilsLogging.getLogger)(siteConfig.quiet);

    var _getArgs = getArgs();

    var buildType = _getArgs.buildType;

    if (buildType) {
        siteConfig["build-type"] = buildType;
    }

    logger("Source: " + siteConfig.source);
    logger("Destination: " + siteConfig.destination);

    /* Start */
    var startTime = Date.now();

    //Transpile custom builds and custom tasks directory first.
    yield transpileCustomBuildsAndTasks(siteConfig);

    var build = builds[siteConfig["build-type"]] || (yield getCustomBuild(siteConfig));

    if (build) {
        yield build(siteConfig);
    } else {
        throw new Error("Build named " + siteConfig["build-type"] + " was not found.");
    }

    var endTime = Date.now();
    logger("Total " + (endTime - startTime) / 1000 + " seconds.");
});

var getSiteConfig = _asyncToGenerator(function* (source, destination) {
    var siteConfig = {};

    var configFilePath = argv.config ? _path2["default"].join(source, argv.config) : (yield _utilsFs2["default"].exists(_path2["default"].join(source, "config.json"))) ? _path2["default"].join(source, "config.json") : _path2["default"].join(source, "config.yml");

    siteConfig = yield (0, _utilsFileReader2["default"])(configFilePath);
    siteConfig.mode = siteConfig.mode || "default";

    var modeSpecificConfigDefaults = configurations[siteConfig.mode].loadDefaults(source, destination);
    var defaults = _utilsConfig2["default"].getFullyQualifiedProperties(modeSpecificConfigDefaults);

    var setter = _utilsConfig2["default"].getValueSetter(siteConfig);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = defaults[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var args = _step.value;

            setter.apply(null, args);
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
                _iterator["return"]();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    _utilsConfig2["default"].commandLineSetter(siteConfig);

    //Store absolute paths for source and destination
    siteConfig.source = _path2["default"].resolve(siteConfig.source);
    siteConfig.destination = _path2["default"].resolve(siteConfig.source, siteConfig.destination);

    //Give modes one chance to update the siteConfig
    if (configurations[siteConfig.mode].updateSiteConfig) configurations[siteConfig.mode].updateSiteConfig(siteConfig);

    return siteConfig;
});

/*
    Transpile dir_custom_builds and dir_custom_tasks
*/
var transpileCustomBuildsAndTasks = _asyncToGenerator(function* (siteConfig) {
    var _arr = [siteConfig["custom-builds-dir"], siteConfig["custom-tasks-dir"]];

    for (var _i = 0; _i < _arr.length; _i++) {
        var dir = _arr[_i];
        var buildRoot = _path2["default"].resolve(siteConfig.source, dir);
        if (yield _utilsFs2["default"].exists(buildRoot)) {
            yield (0, _buildUtilsTasks.runTasks)([{
                name: "transpile-custom-builds-and-plugins",
                plugin: _plugins2["default"].babel,
                options: {
                    source: buildRoot,
                    destination: _path2["default"].resolve(siteConfig.destination, dir),
                    extensions: siteConfig["js-extensions"],
                    blacklist: ["regenerator"]
                }
            }], buildRoot);
        }
    }
});

/*
    Get a build from the siteConfig.dir_custom_builds directory.
    Basically, require(dir_custom_builds/buildName);
*/
var getCustomBuild = _asyncToGenerator(function* (siteConfig) {
    if (siteConfig["custom-builds-dir"] && siteConfig["build-type"]) {
        var fullPath = _path2["default"].resolve(siteConfig.destination, siteConfig["custom-builds-dir"], siteConfig["build-type"] + ".js");
        if (yield _utilsFs2["default"].exists(fullPath)) {
            return require(fullPath);
        }
    }
});

exports["default"] = build;
module.exports = exports["default"];