"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _babelCore = require("babel-core");

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _utilsFs = require("../../../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _utilsLogging = require("../../../utils/logging");

var argv = _optimist2["default"].argv;

/*
    options: {
        destination: string,
        extensions: [string],
        excludedFiles: [string],
        excludedDirectories: [string],
        excludedPatterns: [regex or string],
        excludedWatchPatterns: [regex],
        blacklist: [string],
        quiet: bool
    }
*/

var babel = function babel(name, options) {
    var verboseMode = argv["verbose-" + name];
    var logger = (0, _utilsLogging.getLogger)(options.quiet, name || "babel");

    //defaults
    options.extensions = options.extensions || ["js", "jsx"];
    options.excludedFiles = options.excludedFiles || [];
    options.excludedDirectories = (options.excludedDirectories || []).concat(options.destination);
    options.excludedPatterns = options.excludedPatterns || [];
    options.blacklist = options.blacklist || [];
    options.excludedWatchPatterns = (options.excludedWatchPatterns || []).map(function (p) {
        return new RegExp(p);
    });

    return function () {
        var extensions = options.extensions.map(function (e) {
            return "*." + e;
        });

        var excluded = options.excludedDirectories.map(function (dir) {
            return "!" + dir + "/";
        }).concat(options.excludedFiles.map(function (e) {
            return "!" + e;
        })).concat(options.excludedPatterns.map(function (e) {
            return { exclude: e.exclude, regex: new RegExp(e.regex) };
        }));

        var transpiledFiles = [];

        this.watch(extensions.concat(excluded), _asyncToGenerator(function* (filePath, ev, match) {
            if (!options.excludedWatchPatterns.some(function (regex) {
                return regex.test(filePath);
            })) {
                transpiledFiles.push(filePath);

                //Make the output dir, if it doesn't exist
                var outputPath = _utilsFs2["default"].changeExtension(_path2["default"].join(options.destination, filePath), [{ to: "js", from: options.extensions }]);
                yield _utilsFs2["default"].ensureDirExists(outputPath);
                var contents = yield _utilsFs2["default"].readFile(filePath);

                var result = (0, _babelCore.transform)(contents, { blacklist: options.blacklist });
                yield _utilsFs2["default"].writeFile(outputPath, result.code);

                if (verboseMode) {
                    logger(filePath + " -> " + outputPath);
                }
            }
        }), name, options.dependencies || []);

        this.onComplete(_asyncToGenerator(function* () {
            logger("Rewrote " + transpiledFiles.length + " files");
        }));
    };
};

exports["default"] = babel;
module.exports = exports["default"];