"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _nodefuncPromisify = require("nodefunc-promisify");

var _nodefuncPromisify2 = _interopRequireDefault(_nodefuncPromisify);

var _utilsFs = require("../../../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _utilsConfig = require("../../../utils/config");

var _utilsLogging = require("../../../utils/logging");

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var _browserify = require("browserify");

var _browserify2 = _interopRequireDefault(_browserify);

var _babelify = require("babelify");

var _babelify2 = _interopRequireDefault(_babelify);

var _exposify = require("exposify");

var _exposify2 = _interopRequireDefault(_exposify);

var argv = _optimist2["default"].argv;

/*
    options: {
        source: string,
        destination: string,
        clientBuildDirectory: string,
        appEntryPoint: string,
        bundleName: string,
        jsExtensions: [string],
        debug: bool,
        globalModules: [string],
        excludedModules: [string],
        browserBuildFileSuffix: string,
        browserReplacedFileSuffix: string,
        blacklist: [string],
        excludedFiles: [string],
        excludedDirectories: [string],
        excludedPatterns: [regex or string],
        excludedWatchPatterns = [regex],
        quiet: bool
    }
*/

var buildClient = function buildClient(name, options) {
    var verboseMode = argv["verbose-" + name];
    var logger = (0, _utilsLogging.getLogger)(options.quiet, name || "build-browser-app");

    //defaults
    options.jsExtensions = options.jsExtensions || ["js", "jsx", "json"];
    options.excludedFiles = options.excludedFiles || [];
    options.excludedDirectories = (options.excludedDirectories || []).concat(options.destination);
    options.excludedPatterns = options.excludedPatterns || [];
    options.blacklist = options.blacklist || [];
    options.excludedWatchPatterns = options.excludedWatchPatterns || [];

    var excludedWatchPatterns = options.excludedWatchPatterns.map(function (r) {
        return new RegExp(r);
    });

    //Copy file into destDir
    var copyFile = _asyncToGenerator(function* (filePath, root) {
        //Get the relative filePath by removing the monitored directory (options.source)
        var originalPath = _path2["default"].join(root, filePath);
        var clientPath = _path2["default"].join(options.destination, options.clientBuildDirectory, filePath);
        //We might have some jsx files. Switch extension to js.
        var pathWithFixedExtension = _utilsFs2["default"].changeExtension(clientPath, options.changeExtensions);

        if (verboseMode) {
            logger("Copying " + filePath + " to " + pathWithFixedExtension);
        }

        yield _utilsFs2["default"].copyFile(originalPath, pathWithFixedExtension, { createDir: true });
    });

    return function () {
        var jsExtensions = options.jsExtensions.map(function (e) {
            return "*." + e;
        });

        var excluded = options.excludedDirectories.map(function (dir) {
            return "!" + dir + "/";
        }).concat(options.excludedFiles.map(function (e) {
            return "!" + e;
        })).concat(options.excludedPatterns.map(function (e) {
            return { exclude: e.exclude, regex: new RegExp(e.regex) };
        }));

        var clientSpecificFiles = [];

        this.watch(jsExtensions.concat(excluded), _asyncToGenerator(function* (filePath, ev, matches) {
            if (!excludedWatchPatterns.some(function (regex) {
                return regex.test(filePath);
            })) {
                var clientFileRegex = new RegExp(options.browserBuildFileSuffix + ".(js|json)$");

                if (clientFileRegex.test(filePath)) {
                    if (verboseMode) {
                        logger("Found client-specific file " + filePath);
                    }
                    clientSpecificFiles.push(filePath);
                }

                yield copyFile(filePath, this.root);
            } else {
                if (verboseMode) {
                    logger("Skipped " + filePath);
                }
            }
        }), name, options.dependencies || []);

        /*
            Rules:
                1. In the client build, filename~client.js will be moved to filename.js
                2. Original filename.js will then be renamed filename_base.js (_base is configurable via options.browserReplacedFileSuffix)
                3. filename~client.js will longer exist, since it was moved.
                 The same rules apply for "dev", "test" and other builds.
        */
        var replaceFiles = _asyncToGenerator(function* (files) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var file = _step.value;

                    var filePath = _path2["default"].join(options.destination, options.clientBuildDirectory, file);

                    var extension = /\.js$/.test(file) ? "js" : "json";
                    var regex = new RegExp(options.browserBuildFileSuffix + "\\." + extension + "$");

                    var original = filePath.replace(regex, "." + extension);
                    if (yield _utilsFs2["default"].exists(original)) {
                        var renamed = original.replace(/\.js$/, options.browserReplacedFileSuffix + "." + extension);

                        if (verboseMode) {
                            logger("Moving original " + original + " to " + renamed);
                        }

                        var originalContents = yield _utilsFs2["default"].readFile(original);
                        yield _utilsFs2["default"].writeFile(renamed, originalContents);
                    }

                    var overriddenContents = yield _utilsFs2["default"].readFile(filePath);
                    yield _utilsFs2["default"].writeFile(original, overriddenContents);

                    //Remove abc~client.js and abc~dev.js, as the case may be.
                    if (verboseMode) {
                        logger("Moved " + filePath + " to " + original);
                    }

                    yield _utilsFs2["default"].remove(filePath);

                    if (verboseMode) {
                        logger("Deleted " + filePath);
                    }
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator["return"]) {
                        _iterator["return"]();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        });

        /*
            Create the client and dev builds with browserify.
            Take the entry point from options, which defaults to app.js
        */
        var browserifyFiles = _asyncToGenerator(function* () {
            var entry = _path2["default"].join(options.destination, options.clientBuildDirectory, options.appEntryPoint);
            var output = _path2["default"].join(options.destination, options.clientBuildDirectory, options.bundleName);

            if (verboseMode) {
                logger("Browserify started: entry is " + entry);
            }

            var b = (0, _browserify2["default"])([entry], { debug: options.debug });

            options.excludedModules.concat(Object.keys(options.globalModules)).forEach(function (e) {
                b = b.external(e);
            });

            var r = b.transform(_babelify2["default"].configure({ blacklist: options.blacklist }), { global: true }).transform(_exposify2["default"], { expose: options.globalModules, global: true }).bundle().pipe(_fs2["default"].createWriteStream(output));

            yield (0, _nodefuncPromisify2["default"])(function (cb) {
                r.on("finish", cb);
            })();
        });

        this.onComplete(_asyncToGenerator(function* () {
            //Make the client build
            yield replaceFiles(clientSpecificFiles);
            yield browserifyFiles();
            logger("Wrote " + options.bundleName);
            clientSpecificFiles = [];
        }));
    };
};

exports["default"] = buildClient;
module.exports = exports["default"];