"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _frontMatter = require("front-matter");

var _frontMatter2 = _interopRequireDefault(_frontMatter);

var _doLayout = require("./do-layout");

var _doLayout2 = _interopRequireDefault(_doLayout);

var _utilsFs = require("../../../../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

exports["default"] = function (siteConfig) {

    GLOBAL.site.collections = [];

    var getMakePath = function getMakePath(collection) {
        return function (filePath, page) {
            var permalink = page.permalink || collection.permalink || siteConfig.jekyll.permalink;
            var dir = _path2["default"].dirname(filePath);
            var extension = _path2["default"].extname(filePath);
            var basename = _path2["default"].basename(filePath, extension);

            if (/\/$/.test(permalink)) permalink += "index.html";

            return permalink === "pretty" ? _path2["default"].join(dir, basename, "index.html") : _path2["default"].join(dir, basename + ".html");
        };
    };

    var makePostPath = function makePostPath(filePath, page) {
        var permalink = page.permalink || siteConfig.jekyll.permalink;

        var dir = _path2["default"].dirname(filePath);
        var extension = _path2["default"].extname(filePath);
        var basename = _path2["default"].basename(filePath, extension);

        var _basename$split = basename.split("-");

        var _basename$split2 = _toArray(_basename$split);

        var year = _basename$split2[0];
        var month = _basename$split2[1];
        var day = _basename$split2[2];

        var titleArr = _basename$split2.slice(3);

        var placeholders = {
            year: year,
            month: month,
            day: day,
            title: titleArr.join("-"),
            imonth: parseInt(month).toString(),
            iday: parseInt(day).toString(),
            short_year: parseInt(year) - parseInt(parseInt(year) / 1000) * 1000,
            categories: page.categories ? page.categories.split(/\s+/).join("/") : ""
        };

        var parsePlaceholders = function parsePlaceholders(permalink) {
            for (var key in placeholders) {
                var regex = new RegExp(":\\b" + key + "\\b");
                permalink = permalink.replace(regex, placeholders[key]);
            }
            return permalink.replace(/^\/*/, "");
        };

        if (/\/$/.test(permalink)) permalink += "index.html";

        return permalink === "pretty" ? parsePlaceholders("/:categories/:year/:month/:day/:title/index.html") : permalink === "date" ? parsePlaceholders("/:categories/:year/:month/:day/:title.html") : permalink === "none" ? parsePlaceholders("/:categories/:title.html") : parsePlaceholders(permalink);
    };

    var makePagePath = function makePagePath(filePath, page) {
        var permalink = page.permalink || siteConfig.permalink;

        var dir = _path2["default"].dirname(filePath);
        var extension = _path2["default"].extname(filePath);
        var basename = _path2["default"].basename(filePath, extension);

        return permalink === "pretty" ? _path2["default"].join(dir, basename, "index.html") : _path2["default"].join(dir, basename + ".html");
    };

    var fn = _asyncToGenerator(function* () {
        for (var collectionName in siteConfig.collections) {
            var collection = siteConfig.collections[collectionName];
            if (collection.output) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = GLOBAL.site.data[collectionName][Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var item = _step.value;

                        var makePath = collectionName === "posts" ? makePostPath : collectionName === "pages" ? makePagePath : getMakePath(collection);

                        //If we don't have a filename, we don't need to process it individually.
                        if (item.__filePath) {
                            yield (0, _doLayout2["default"])(item, item.__filePath, collection.layout || "default", makePath, siteConfig);
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator["return"]) {
                            _iterator["return"]();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            }
        }
    });

    return { build: false, fn: fn };
};

module.exports = exports["default"];