"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _less = require("less");

var _less2 = _interopRequireDefault(_less);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _utilsFs = require("../../../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _nodefuncPromisify = require("nodefunc-promisify");

var _nodefuncPromisify2 = _interopRequireDefault(_nodefuncPromisify);

var _utilsLogging = require("../../../utils/logging");

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var lessc = (0, _nodefuncPromisify2["default"])(_less2["default"].render.bind(_less2["default"]));
var argv = _optimist2["default"].argv;

/*
    options: {
        files: [string],
        destination: string,
        directories: [string],
        quiet: bool
    }
*/
var compileLess = function compileLess(name, options) {
    var verboseMode = argv["verbose-" + name];
    var logger = (0, _utilsLogging.getLogger)(options.quiet, name || "less");

    //defaults
    var directories = options.files.map(function (f) {
        return _path2["default"].dirname(f);
    }).filter(function (item, i, ar) {
        return ar.indexOf(item) === i;
    });

    options.excludedDirectories = options.excludedDirectories || [];

    var extensions = directories.map(function (dir) {
        return dir + "/*.less";
    });
    var excluded = options.excludedDirectories.map(function (dir) {
        return "!" + dir + "/";
    });

    return function () {
        var mustCompile = false;

        this.watch(extensions.concat(excluded), _asyncToGenerator(function* (filePath, ev, match) {
            mustCompile = true;
        }));
        this.onComplete(_asyncToGenerator(function* () {
            if (mustCompile) {
                mustCompile = false;
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = options.files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var filePath = _step.value;

                        var outputPath = _path2["default"].join(options.destination, filePath).replace(/\.less$/, ".css");
                        logger("Compiling " + filePath + " to " + outputPath);

                        var outputDir = _path2["default"].dirname(outputPath);
                        if (!(yield _utilsFs2["default"].exists(outputDir))) {
                            yield _utilsFs2["default"].mkdirp(outputDir);
                        }
                        var contents = yield _utilsFs2["default"].readFile(filePath);
                        if (contents) {
                            var result = yield lessc(contents, { paths: [_path2["default"].dirname(_path2["default"].join(options.source, filePath))] });
                            yield _utilsFs2["default"].writeFile(outputPath, result.css);
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator["return"]) {
                            _iterator["return"]();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            }
        }), name, options.dependencies || []);
    };
};

exports["default"] = compileLess;
module.exports = exports["default"];