"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _frontMatter = require("front-matter");

var _frontMatter2 = _interopRequireDefault(_frontMatter);

var _jsYaml = require("js-yaml");

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _utilsFs = require("../../../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _utilsFileReader = require("../../../utils/file-reader");

var _utilsFileReader2 = _interopRequireDefault(_utilsFileReader);

var _utilsLogging = require("../../../utils/logging");

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var argv = _optimist2["default"].argv;

/*
    options: {
        dataVariable: object,
        collections: {
            name1: { dir: string },
            name2: { dir: string }
        },
        collectionRootDirectory: string,
        dataDirectories: [string],
        scavengeCollection: string,
        excludedDirectories: [string],
        excludedFiles: [string],
        markdownExtensions: [string]
    }
*/
var loadStaticData = function loadStaticData(name, options) {
    var verboseMode = argv["verbose-" + name];
    var logger = (0, _utilsLogging.getLogger)(options.quiet, name || "load-static-data");

    var data = options.data;

    //Add a watch for each collection.
    var getCollectionLoader = function getCollectionLoader(collection) {
        return _asyncToGenerator(function* (filePath) {
            var extension = _path2["default"].extname(filePath);

            try {
                logger("Loading " + filePath + " into " + collection);

                var record = yield (0, _utilsFileReader2["default"])(filePath, { markdown: options.markdownExtensions });
                record.__filePath = filePath;

                if (record) {
                    data[collection].push(record);
                }
            } catch (ex) {
                logger(ex);
            }
        });
    };

    return function () {
        var _this = this;

        //Data directories
        this.watch(["yaml", "yml", "json"].map(function (ext) {
            return options.dataDirectories.map(function (dir) {
                return dir + "/*." + ext;
            });
        }).reduce(function (a, b) {
            return a.concat(b);
        }), _asyncToGenerator(function* (filePath) {
            var extension = _path2["default"].extname(filePath);

            try {
                var records = yield (0, _utilsFileReader2["default"])(filePath);
                var filename = _path2["default"].basename(filePath, extension);

                logger("Loading " + filePath + " into " + filename);

                if (records && records.length) {
                    data[filename] = data[filename] ? data[filename].concat(records) : records;
                }
            } catch (ex) {
                logger(ex);
            }
        }), name, options.dependencies || []);

        //Check the collection directories
        for (var collectionName in options.collections) {
            data[collectionName] = [];
            var collection = options.collections[collectionName];
            if (collection.dir) {
                (function () {
                    var collectionDir = options.collectionRootDirectory ? _path2["default"].combine(options.collectionRootDirectory, collection.dir) : collection.dir;
                    _this.watch(options.markdownExtensions.concat(["json"]).map(function (ext) {
                        return collectionDir + "/*." + ext;
                    }), getCollectionLoader(collectionName), name + "-collection-loader", options.collectionLoaderDependencies || []);
                })();
            }
        }

        //If scavenging is on, we need to pick up md and json files outside
        //  collection and data_dir and push them into the scavenge collection.
        if (options.scavengeCollection) {
            data[options.scavengeCollection] = [];

            var collectionsAndDataDirs = Object.keys(options.collections).map(function (coll) {
                return options.collections[coll].dir;
            }).filter(function (item) {
                return item;
            }).concat(options.dataDirectories).map(function (dir) {
                return "!" + dir + "/";
            });

            var exclusions = options.excludedDirectories.map(function (e) {
                return "!" + e + "/";
            }).concat(options.excludedFiles.map(function (e) {
                return "!" + e;
            })).concat(collectionsAndDataDirs);

            var filePatterns = options.markdownExtensions.concat(["json"]).map(function (ext) {
                return "*." + ext;
            });

            this.watch(filePatterns.concat(exclusions), getCollectionLoader(options.scavengeCollection), name + "-scavenge-collection", options.scavengeCollectionDependencies || []);
        }
    };
};

exports["default"] = loadStaticData;
module.exports = exports["default"];