"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var _crankshaftTools = require("crankshaft-tools");

var _crankshaftTools2 = _interopRequireDefault(_crankshaftTools);

var _utilsFs = require("../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _utilsLogging = require("../utils/logging");

var _utilsCli = require("../utils/cli");

var _utilsCli2 = _interopRequireDefault(_utilsCli);

var argv = _optimist2["default"].argv;

var printSyntax = function printSyntax(msg) {
    if (msg) {
        (0, _utilsLogging.print)("Error: " + msg);
    }
    (0, _utilsLogging.print)("Usage: fora install <template_name> [--git]");
    process.exit();
};

var getArgs = function getArgs() {
    var args = _utilsCli2["default"].getArgs();

    if (args.length < 4) {
        printSyntax();
    }
    /* params */
    var template = args[3];
    return { template: template };
};

var install = _asyncToGenerator(function* () {
    var HOME_DIR = process.env.HOME || process.env.HOMEPATH || process.env.USERPROFILE;
    var templatesDir = _path2["default"].join(HOME_DIR, ".fora", "templates");
    var nodeModulesDir = _path2["default"].join(templatesDir, "node_modules");

    //Make sure ~/.fora/templates/node_modules exists
    if (!(yield _utilsFs2["default"].exists(nodeModulesDir))) {
        yield _utilsFs2["default"].mkdirp(nodeModulesDir);
    }

    var _shellExec = _crankshaftTools2["default"].process.spawn({ stdio: "inherit" });
    var shellExec = _asyncToGenerator(function* (cmd) {
        yield _shellExec("sh", ["-c", cmd]);
    });

    if (argv.git) {
        process.chdir(nodeModulesDir);
        var templateUrl = getArgs().template;
        var urlParts = templateUrl.split("/");
        var template = urlParts[urlParts.length - 1];
        var destDir = _path2["default"].join(nodeModulesDir, template);
        if (yield _utilsFs2["default"].exists(destDir)) {
            (0, _utilsLogging.print)(destDir + " exists. Will git pull.");
            process.chdir(template);
            yield shellExec("git pull");
        } else {
            (0, _utilsLogging.print)("Cloning to " + destDir + ".");
            yield shellExec("git clone " + templateUrl);
            process.chdir(template);
            yield shellExec("npm install");
        }
    } else {
        var _getArgs = getArgs();

        var template = _getArgs.template;

        (0, _utilsLogging.print)("Installing " + template + " with npm. This make take a few minutes.");
        process.chdir(templatesDir);
        yield shellExec("npm install " + template);
    }
});

exports["default"] = install;
module.exports = exports["default"];