"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { var callNext = step.bind(null, "next"); var callThrow = step.bind(null, "throw"); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(callNext, callThrow); } } callNext(); }); }; }

var _crankshaftTools = require("crankshaft-tools");

var _crankshaftTools2 = _interopRequireDefault(_crankshaftTools);

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _utilsFs = require("../utils/fs");

var _utilsFs2 = _interopRequireDefault(_utilsFs);

var _utilsLogging = require("../utils/logging");

var _utilsCli = require("../utils/cli");

var _utilsCli2 = _interopRequireDefault(_utilsCli);

var argv = _optimist2["default"].argv;

var printSyntax = function printSyntax(msg) {
    if (msg) {
        (0, _utilsLogging.print)("Error: " + msg);
    }
    (0, _utilsLogging.print)("Usage: fora new <template> <project_name> [-d <destination>] [--recreate]");
    process.exit();
};

var getArgs = function getArgs() {
    var args = _utilsCli2["default"].getArgs();

    if (args.length < 5) {
        printSyntax();
    }
    /* params */
    var template = args[3];
    var name = args[4].trim().replace(/\s+/g, '-').toLowerCase();
    return { template: template, name: name };
};

/*
    Copy files from the template directory to the destination directory.
*/
var copyTemplateFiles = _asyncToGenerator(function* () {
    var logger = (0, _utilsLogging.getLogger)(argv.quiet || false);

    var _getArgs = getArgs();

    var template = _getArgs.template;
    var name = _getArgs.name;

    var destination = _path2["default"].join(argv.d || argv.destination || "./", name);
    var destinationExists = yield _utilsFs2["default"].exists(destination);

    //Make sure the directory is empty or the recreate flag is on
    if (destinationExists && !argv.recreate) {
        (0, _utilsLogging.print)("Conflict: " + _path2["default"].resolve(destination) + " is not empty. Delete the directory manually or use --recreate.");
    } else {
        yield* (function* () {

            if (destinationExists) {
                if (argv.recreate) {
                    (0, _utilsLogging.print)("Deleting " + destination);
                    yield _utilsFs2["default"].remove(destination);
                    yield _utilsFs2["default"].mkdirp(destination);
                }
            } else {
                yield _utilsFs2["default"].mkdirp(destination);
            }

            //Copy template
            var _shellExec = _crankshaftTools2["default"].process.spawn({ stdio: "inherit" });
            var shellExec = _asyncToGenerator(function* (cmd) {
                yield _shellExec("sh", ["-c", cmd]);
            });
            var exec = _crankshaftTools2["default"].process.exec();

            var templatePath = yield resolveTemplatePath(template);
            logger("Copying " + templatePath + " -> " + destination);
            yield _utilsFs2["default"].copyRecursive(templatePath, destination, { forceDelete: true });

            //Install npm dependencies.
            var curdir = yield exec("pwd");
            process.chdir(destination);
            yield shellExec("npm install");
            process.chdir(curdir);

            //Let's overwrite package.json
            var packageJsonPath = _path2["default"].join(destination, "package.json");
            var packageJson = yield _utilsFs2["default"].readFile(packageJsonPath);
            var packageInfo = JSON.parse(packageJson);
            packageInfo.name = name;
            packageInfo.description = argv["set-package-description"] || "Description for your project";
            packageInfo.version = argv["set-package-version"] || "0.0.1";
            packageInfo.homepage = argv["set-package-homepage"] || "Your home page";
            packageInfo.repository = {
                type: argv["set-package-repository-type"] || "git",
                url: argv["set-package-repository-url"] || "https://example.com/your/repository/path"
            };
            packageInfo.author = argv["set-package-author"] || "Your name";
            packageInfo.email = argv["set-package-email"] || "youremail@example.com";
            packageInfo.bugs = argv["set-package-bugs"] || "http://www.example.com/bugzilla/your-project";
            yield _utilsFs2["default"].writeFile(_path2["default"].join(destination, "package.json"), JSON.stringify(packageInfo, null, 4));

            (0, _utilsLogging.print)("New " + template + " site installed in " + _path2["default"].resolve(destination) + ".");
        })();
    }
});

/*
    Search paths are:
        a) Current node_modules directory
        b) ~/.fora/templates/node_modules
*/
var resolveTemplatePath = _asyncToGenerator(function* (name) {
    var templateName = /^fora-template-/.test(name) ? name : "fora-template-" + name;

    //Current node_modules_dir
    var node_modules_templatePath = _path2["default"].resolve(GLOBAL.__libdir, "../node_modules", name);
    var node_modules_prefixedTemplatePath = _path2["default"].resolve(GLOBAL.__libdir, "../node_modules", "fora-template-" + name);

    /*
        Templates can also be under
            ~/.fora/templates/example-template
            ~/.fora/templates/node_modules/example-template
    */
    var HOME_DIR = process.env.HOME || process.env.HOMEPATH || process.env.USERPROFILE;
    var HOME_templatePath = _path2["default"].resolve(HOME_DIR + "/.fora/templates", name);
    var HOME_prefixedTemplatePath = _path2["default"].resolve(HOME_DIR + "/.fora/templates", "fora-template-" + name);
    var HOME_node_modules_templatePath = _path2["default"].resolve(HOME_DIR + "/.fora/templates/node_modules", name);
    var HOME_node_modules_prefixedTemplatePath = _path2["default"].resolve(HOME_DIR + "/.fora/templates/node_modules", "fora-template-" + name);

    var paths = [node_modules_templatePath, node_modules_prefixedTemplatePath, HOME_templatePath, HOME_prefixedTemplatePath, HOME_node_modules_templatePath, HOME_node_modules_prefixedTemplatePath];

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = paths[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var templatePath = _step.value;

            if (yield _utilsFs2["default"].exists(templatePath)) {
                return templatePath;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
                _iterator["return"]();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    throw new Error("Template \"" + name + "\" or \"fora-template-" + name + "\" was not found.");
});

exports["default"] = copyTemplateFiles;
module.exports = exports["default"];